/* Copyright (c) 2023 Renmin University of China
RMDB is licensed under Mulan PSL v2.
You can use this software according to the terms and conditions of the Mulan PSL v2.
You may obtain a copy of Mulan PSL v2 at:
        http://license.coscl.org.cn/MulanPSL2
THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
See the Mulan PSL v2 for more details. */

#pragma once

#include "common/common.h"
#include "execution_defs.h"
#include "index/ix.h"
#include "system/sm.h"

class AbstractExecutor {
   public:
    Rid _abstract_rid;

    Context *context_;

    virtual ~AbstractExecutor() = default;

    virtual int tupleLen() const = 0;

    virtual const std::vector<ColMeta> &cols() const {
        static std::vector<ColMeta> cols;
        return cols;
    }
    // virtual const std::vector<ColMeta> &cols() const {
    //     std::vector<ColMeta> *_cols = nullptr;
    //     return *_cols;
    // };

    virtual std::string getType() = 0;
    // virtual std::string getType() {return "AbstractExecutor";}

    virtual void beginTuple() = 0;

    virtual void nextTuple() = 0;

    virtual bool is_end() const = 0;

    virtual Rid &rid() = 0;

    // 不可随便调用，seq scan方法中使用了move语义，仅能调用一次
    virtual std::unique_ptr<RmRecord> Next() = 0;

    // virtual ColMeta get_col_offset(const TabCol &target) =0;
    /**
     * @description: 获取 tabcol 对应的 ColMeta
     */
    virtual ColMeta get_col_offset(const TabCol &target) { return ColMeta(); };

    std::vector<ColMeta>::const_iterator get_col(const std::vector<ColMeta> &rec_cols, const TabCol &target) {
        auto pos = std::find_if(rec_cols.begin(), rec_cols.end(), [&](const ColMeta &col) {
            return col.tab_name == target.tab_name && col.name == target.col_name;
        });
        if (pos == rec_cols.end()) {
            throw ColumnNotFoundError(target.tab_name + '.' + target.col_name);
        }
        return pos;
    }
};
